/*
Rocket Flight Logger ver 1.6
 Copyright Boris du Reau 2012-2013
 
 The following is a datalogger for logging rocket flight.
 So far it can log the rocket altitude during the flight.
 
 
 This is using a BMP085 presure sensor and an Atmega 328
 The following record the flight in an EEPROM
 
 
 For the BMP085 pressure sensor
 Connect VCC of the BMP085 sensor to 5.0V! make sure that you are using the 5V sensor (GY-65 model)
 Connect GND to Ground
 Connect SCL to i2c clock - on 328 Arduino Uno/Duemilanove/etc thats Analog 5
 Connect SDA to i2c data - on 328 Arduino Uno/Duemilanove/etc thats Analog 4
 EOC is not used, it signifies an end of conversion
 
 For the EEPROM 24LC512-I/P - SERIAL EEPROM 512K,24LC512, DIP8
 
 Number Name ConnectTo     Note
 1       a0     GND
 2       a1     GND
 3       a2     GND
 4       GND    GND
 5       SDA    SDA         2kOhm PullUp
 6       SCL    SCL
 7       WP     GND
 8       VCC    (+3v3 ... +5V0)
 
 The time to write in the EEPROM is 10ms
 
 
 Major changes on version 1.5
 Altimeter configuration has been softcoded
 
 Major changes on version 1.6
 The code is now splitted in different files for better readability
 */


#include <Wire.h> //I2C library
#include <Adafruit_BMP085.h>
//used for writing in the microcontroleur lib
#include <EEPROM.h>
#include "kalman.h"
#include "i2c_eeprom.h"
#include "beepfunc.h"
#include "config.h"

//////////////////////////////////////////////////////////////////////
// Global variables
//////////////////////////////////////////////////////////////////////
int mode = 0; //0 = read; 1 = write;
Adafruit_BMP085 bmp;
long endAddress = 65536;

//ground level altitude
long initialAltitude;
long liftoffAltitude;
long lastAltitude;
//current altitude
long currAltitude;
//Apogee altitude
long apogeeAltitude;
long mainAltitude;
boolean liftOff= false;
unsigned long initialTime;

boolean FastReading = false;
boolean mainHasFired = false;

//nbr of measures to do so that we are sure that apogee has been reached 
unsigned long measures=5;
unsigned long mainDeployAltitude;

// pin used by the jumpers
const int pinAltitude1 = 8;
const int pinAltitude2 = 7;

//soft configuration
boolean softConfigValid =false;

const int pinApogeeContinuity = 10;
const int pinMainContinuity = 11;
const int pinOut3Continuity = 16;

//#define NBR_MEASURE_APOGEE 5
float FEET_IN_METER =1;
boolean canRecord = true;

//================================================================
// Start program
//================================================================
void setup()
{
  int val = 0;     // variable to store the read value
  int val1 = 0;     // variable to store the read value

  //pyro out 1
  pinApogee = pyroOut1;
  //pyro out 2
  pinMain= pyroOut2;
  //pyro out 3
  pinOut3= pyroOut3;


  // Read altimeter softcoded configuration
  softConfigValid = readAltiConfig();
  // check if configuration is valid 
  if (!softConfigValid)
  {
    //default values
    defaultConfig();
    /*int i;
     for( i=0; i<sizeof(config); i++ ) {
     EEPROM.write(CONFIG_START+i, *((char*)&config + i));
     }*/
    writeConfigStruc();
  }
  //Assign Pyro output to main, apogee or airstart
  getOutPin(config.outPut1);
  getOutPin(config.outPut2);
  getOutPin(config.outPut3);


  //set main altitude (if in feet convert to metrics)
  if(config.unit ==0)
    FEET_IN_METER =1;
  else
    FEET_IN_METER =3.28084 ;

  mainDeployAltitude= int(config.mainAltitude /FEET_IN_METER); 
  //beepFrequency
  beepingFrequency = config.beepingFrequency;

  //init Kalman filter
  KalmanInit();

  // initialise the connection
  Wire.begin(); 

  //You can change the baud rate here 
  //and change it to 57600, 115200 etc..
  Serial.begin(9600);

  pinMode(A0, INPUT);

  //Presure Sensor Initialisation
  bmp.begin( BMP085_STANDARD);
  //our drogue has not been fired
  apogeeHasFired=false;
  mainHasFired=false;

  Serial.print(F("Start program\n"));

  //Initialise the output pin
  pinMode(pinApogee, OUTPUT);
  pinMode(pinMain, OUTPUT);
  pinMode(pinOut3, OUTPUT);
  pinMode(pinSpeaker, OUTPUT);

  pinMode(pinAltitude1, INPUT);
  pinMode(pinAltitude2, INPUT);

  pinMode(pinApogeeContinuity, INPUT);
  pinMode(pinMainContinuity, INPUT);
  pinMode(pinOut3Continuity, INPUT);

  //Make sure that the output are turned off
  digitalWrite(pinApogee, LOW);
  digitalWrite(pinMain, LOW);
  digitalWrite(pinOut3, LOW);
  digitalWrite(pinSpeaker, LOW);

  //initialisation give the version of the altimeter
  //One long beep per major number and One short beep per minor revision
  //For example version 1.2 would be one long beep and 2 short beep
  beepAltiVersion(MAJOR_VERSION,MINOR_VERSION);

  //number of measures to do to detect Apogee
  measures = 5; //NBR_MEASURE_APOGEE;

  if (!softConfigValid)
  {
    //initialise the deployement altitude for the main 
    mainDeployAltitude = 100;

    // On the Alti duo when you close the jumper you set it to 1
    // val is the left jumper and val1 is the right jumper
    //as of version 1.4 only use the jumper if no valid softconfiguration

    val = digitalRead(pinAltitude1); 
    val1 = digitalRead(pinAltitude2);  
    if(val == 0 && val1 ==0)
    {
      mainDeployAltitude = 50;
    }
    if(val == 0 && val1 ==1)
    {
      mainDeployAltitude = 100;
    }
    if(val == 1 && val1 ==0)
    {
      mainDeployAltitude = 150;
    }
    if(val == 1 && val1 ==1)
    {
      mainDeployAltitude = 200;
    }
  }
  // let's do some dummy altitude reading
  // to initialise the Kalman filter
  for (int i=0; i<50; i++){
    KalmanCalc(bmp.readAltitude());
  }

  //let's read the launch site altitude
  long sum = 0;
  for (int i=0; i<10; i++){
    sum += KalmanCalc(bmp.readAltitude());
    delay(50); 
  }
  initialAltitude = (sum / 10.0);
  lastAltitude = 0;//initialAltitude; 
  liftoffAltitude = 20;

  getFlightList();

 // check if eeprom is full
 if(isEepromFull())
   canRecord == false;
}


//================================================================
// Main loop which call the menu
//================================================================
void loop()
{
  MainMenu();
}  



void recordAltitude()
{
  boolean exit=false;
  boolean apogeeReadyToFire = false;
  boolean mainReadyToFire = false;
  unsigned long apogeeStartTime =0;
  unsigned long mainStartTime =0;

  while (exit == false) 
  {

    //read current altitude
    currAltitude = (KalmanCalc(bmp.readAltitude())- initialAltitude);
    if (( currAltitude > liftoffAltitude) == true && liftOff == false)
    {
      liftOff = true;
      // save the time
      initialTime =millis();
    }  
    if(liftOff)
    {
      Serial.println(F("we have lift off\n"));
      if (canRecord)
      {
      //Save start address
      fileA[currentFileNbr].startaddress = startMemaddress;
      }
      unsigned long prevTime=0;
      // loop until we have reach an altitude of 3 meter
      while(currAltitude > 3 && mainHasFired==false)
      {
        unsigned long currentTime;
        unsigned long diffTime;

        byte currentTimeMSB = B00000000;
        byte currentTimeLSB = B00000000;

        byte currAltitudeMSB = B00000000;
        byte currAltitudeLSB = B00000000;

        currAltitude = (KalmanCalc(bmp.readAltitude())- initialAltitude);

        currentTime = millis()- initialTime;
        diffTime = currentTime - prevTime;
        prevTime = currentTime;
        //currentTimeLSB = currentTime;
        //currentTimeMSB = currentTime >> 8;
        if (canRecord)
      {
        currentTimeLSB = diffTime;
        currentTimeMSB = diffTime >> 8;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currentTimeMSB);
        currentMemaddress++;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currentTimeLSB);
        currentMemaddress++;

        currAltitudeLSB = currAltitude;
        currAltitudeMSB = currAltitude >> 8;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currAltitudeMSB);
        currentMemaddress++;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currAltitudeLSB);
        currentMemaddress++;
      }
        if (currAltitude < lastAltitude && apogeeHasFired == false)
        {
          measures = measures - 1;
          if (measures == 0)
          {
            //fire drogue
            //digitalWrite(pinApogee, HIGH);
            apogeeReadyToFire = true;
            apogeeStartTime = millis();
            //apogeeHasFired=true;

            apogeeAltitude = currAltitude;
          }  
        }
        else 
        {
          lastAltitude = currAltitude;
          measures = 5;
        }   
        if(apogeeReadyToFire)
        {
          if((millis()-apogeeStartTime) > config.outPut1Delay)
          {
            //fire drogue
            digitalWrite(pinApogee, HIGH);
            apogeeReadyToFire = false;
            apogeeHasFired=true;
          }
        }

        if ((currAltitude  < mainDeployAltitude) && apogeeHasFired == true && mainHasFired==false)
        {
          // Deploy main chute  X meters or feet  before landing...
          digitalWrite(pinApogee, LOW);
          mainReadyToFire = true;
          mainStartTime = millis();
          //digitalWrite(pinMain, HIGH); 
          //mainHasFired=true;
          mainAltitude= currAltitude;     
        }
        if(mainReadyToFire)
        {
          Serial.println("conf delay main" + config.outPut1Delay );
          Serial.println("conf delay" + config.outPut2Delay );
          if((millis()-mainStartTime) > config.outPut2Delay)
          {
            //fire drogue
            Serial.println("firing main");
            digitalWrite(pinMain, HIGH);
            mainReadyToFire = false;
            mainHasFired=true;
          }
        }
      }
      if (canRecord)
      {
      //end loging
      //store start and end address
      Serial.println(F("stop recording\n "));
      //save end address
      fileA[currentFileNbr].endaddress = currentMemaddress -1;
      saveFlightA(currentFileNbr, startMemaddress, currentMemaddress -1);
      }
      exit =true;
      if (mainHasFired)
      {
        
        delay(1000);
        digitalWrite(pinMain, LOW);
        liftOff =false;
      }
    }
  }
}




//================================================================
// Main menu to interpret all the commands sent by the altimeter console
//================================================================
void MainMenu()
{
  char readVal=' ';
  int i=0;

  char commandbuffer[100];

  Serial.println(F("Rocket flight data logger. A maximum of 25 flight can be logged \n"));
  Serial.println(F("Commands are: \n"));
  Serial.println(F("w = record flight \n"));
  Serial.println(F("r (followed by the flight number) = read flight data\n"));
  Serial.println(F("l  = print flight list \n"));
  Serial.println(F("e  = erase all flight data \n"));
  Serial.println(F("c  = toggle continuity on/off \n"));
  Serial.println(F("b  = print alti config \n"));
  Serial.println(F("Enter Command and terminate it by a ; >>\n"));
  i=0;
  readVal=' ';
  while( readVal != ';') 
  {
    if (FastReading == false)
    {
      currAltitude = (KalmanCalc(bmp.readAltitude())- initialAltitude);
      if (( currAltitude > liftoffAltitude) != true)
      {
        continuityCheck(pinApogeeContinuity);
        continuityCheck(pinMainContinuity);
        continuityCheck(pinOut3Continuity);
        delay (500);
      }
      else
      {
        recordAltitude();
      }
      if(apogeeHasFired == true && mainHasFired==true)
      {
        beginBeepSeq();

        if(config.beepingMode ==0)
          beepAltitude(apogeeAltitude* FEET_IN_METER);
        else
          beepAltitudeNew(apogeeAltitude * FEET_IN_METER);
        beginBeepSeq();

        if(config.beepingMode ==0)
          beepAltitude(mainAltitude* FEET_IN_METER);
        else
          beepAltitudeNew(mainAltitude* FEET_IN_METER);
      }
    } 
    //if(Serial.available())
    while(Serial.available())
    {
      readVal = Serial.read();
      if (readVal != ';' )
      {
        if (readVal != '\n')
          commandbuffer[i++] = readVal;
      }
      else
      {
        commandbuffer[i++]='\0';
        break;
      }
    }
  }
  Serial.println((char*)commandbuffer);
  if (commandbuffer[0] == 'e')
  {
    Serial.println(F("Erase\n"));
    i2c_eeprom_erase_fileList();
  }  
  else if (commandbuffer[0] == 'r')  
  {
    char temp[3];
    Serial.println(F("Read flight: "));
    Serial.println( commandbuffer[1]);
    Serial.println( "\n");
    temp[0] = commandbuffer[1];
    if (commandbuffer[2] != '\0')
    {
      temp[1] = commandbuffer[2];
      temp[2] = '\0';
    }
    else
      temp[1] ='\0';

    if(atol(temp)>0)
    {
      printFlight(atol(temp));
    }
    else
      Serial.println(F("not a valid flight"));
  }
  // Recording
  else if (commandbuffer[0] == 'w') 
  {
    Serial.println(F("Recording \n"));
    recordAltitude();
  }
  //Number of flight
  else if (commandbuffer[0] == 'n') 
  {
    Serial.println(F("Number of flight \n"));
    Serial.print(F("n;"));
    Serial.println(getFlightList());
    //recordAltitude();
  }
  else if (commandbuffer[0] == 'l')
  {
    Serial.println(F("Flight List: \n"));
    printFlightList();
  }
  else if (commandbuffer[0] == 'c')
  {
    if (noContinuity == false)
    {
      noContinuity = true;
      Serial.println(F("Continuity off \n"));
    }
    else
    {
      noContinuity = false;
      Serial.println(F("Continuity on \n"));
    }
  }
  else if (commandbuffer[0] == 'a')
  {
    Serial.print(F("$start;\n"));
    //getFlightList()
    int i;
    for (i=1; i<getFlightList()+1;i++)
    {
      printFlightData(i);
    }  
    Serial.print(F("$end;\n"));
  }
  //altimeter config
  else if (commandbuffer[0] == 'b')
  {
    Serial.print(F("$start;\n"));

    printAltiConfig();

    Serial.print(F("$end;\n"));
  }
  else if (commandbuffer[0] == 's')
  {
    writeAltiConfig(commandbuffer);
  }
  //FastReading
  else if (commandbuffer[0] == 'f')
  {
    FastReading = true;
    Serial.print(F("$OK;"));

  }
  //FastReading off
  else if (commandbuffer[0] == 'g')
  {
    FastReading = false;
    Serial.print(F("$OK;"));
  }
   //hello
  else if (commandbuffer[0] == 'h')
  {
    //FastReading = false;
    Serial.print(F("$OK;"));
  }
  else
  {
    Serial.println(F("Unknown command" ));
    Serial.print(F("$UNKNOWN;"));
    Serial.println(commandbuffer[0] + "\n");
  }
}


